// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: https://svn.inet.dkfz-heidelberg.de/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Core/include/mapFastMutexLockSentinel.h $
*/

#ifndef __MAP_FAST_MUTEX_LOCK_SENTINEL_H
#define __MAP_FAST_MUTEX_LOCK_SENTINEL_H

#include "itkSimpleFastMutexLock.h"

#include "mapMacros.h"
#include "mapMAPCoreExports.h"


namespace map
{
	namespace core
	{
		namespace services
		{
			/*! @class FastMutexLockSentinel
			* @brief Helper class for fast locked threadding policies.
			* Used as their sentinel class. Locks the mutex when calling InitializeSentinel()
			* unlocks the mutex when the destructor of the sentinel is called.
			*
			* @ingroup ThreadingPolicies
			*/
			class MAPCore_EXPORT FastMutexLockSentinel
			{
			public:
				typedef itk::SimpleFastMutexLock  MutexType;

				void initializeSentinel(MutexType* pMutex);

				FastMutexLockSentinel();

				~FastMutexLockSentinel();

			private:
				/*! This policy uses this mutex to lock critical code by the policy user.*/
				MutexType* _pMutex;

				FastMutexLockSentinel(const FastMutexLockSentinel&);  //purposely not implemented
				void operator=(const FastMutexLockSentinel&);  //purposely not implemented
			};

		} // end namespace services
	} // end namespace core
} // end namespace map

#endif